const facetBlock = document.querySelector(".UNX-fxd-facet");
const btnEls = document.querySelectorAll(".UNX-facet-trigger");
const searchInput = document.getElementById("unbxdInput");
const searchBtn = document.getElementById("searchBtn");
const priceField = typeof(window.UNBXD_MAPPED_FIELDS) === "object" ? window.UNBXD_MAPPED_FIELDS["price"] : "";
let showFacet = false;
let sortOptions = [];
window.resizeTimer = null;

if (typeof(priceField) !== "undefined" && priceField) {
    /* add sort options if price field dimension mapping is provided */
    sortOptions.push({
        value: `${priceField} desc`,
        text: "Price: High to Low"
    }, {
        value: `${priceField} asc`,
        text: "Price: Low to High",
    });
}

const scrollToTop = () => {
    document.body.scrollTop = 0;
    document.documentElement.scrollTop = 0;
}

const getFields = function() {
    let fields = [window.UNBXD_MAPPED_FIELDS.price, window.UNBXD_MAPPED_FIELDS.title, window.UNBXD_MAPPED_FIELDS.imageUrl];
    let validatedFields = fields.filter(function(item) {
        return item;
    });
    return validatedFields;
};

const toggleMobileFacets = (e) => {
    showFacet = !showFacet;
    const {
        action
    } = e.target.dataset;
    if (action === "applyFacets") {
        scrollToTop();
        window.unbxdSearch.setPageStart(0);
        window.unbxdSearch.getResults();
    }
    if (action === "clearFacets") {
        scrollToTop();
        window.unbxdSearch.clearAllFacets();
        window.unbxdSearch.setPageStart(0);
        window.unbxdSearch.getResults();
    }
    if (showFacet) {
        facetBlock.classList.add("UNX-show-facets")
    } else {
        facetBlock.classList.remove("UNX-show-facets")
    }
}

btnEls.forEach(item => {
    item.addEventListener("click", toggleMobileFacets)
})

window.unbxdSearch = new window.UnbxdSearch({
    siteKey: window.UNBXD_SITE_KEY,
    apiKey: window.UNBXD_API_KEY,
    searchBoxEl: searchInput,
    searchEndPoint: `${window.UNBXD_SEARCH_URL}/`,
    hashMode: false,
    searchTrigger: "click",
    searchButtonEl: searchBtn,
    products: {
        productType: "SEARCH",
        el: document.getElementById("searchResultsWrapper"),
        productAttributes: [
            "title",
            "uniqueId",
            "price",
            "sku",
            "ImageURL",
            "displayPrice",
            "salePrice",
            "sortPrice",
            "productDescription",
            "unbxd_color_mapping",
            "colorName",
            "color",
            ...getFields(),
            priceField
        ],
        attributesMap: window.UNBXD_MAPPED_FIELDS,
        template: function(product, idx, swatchUI, productViewType, products) {
            const {
                description,
                uniqueId,
                imageUrl,
                title,
                price
            } = product;
            const {
                productItemClass
            } = products;
            const imgUrl = Array.isArray(imageUrl) ? imageUrl[0] : imageUrl;
            let imagesUI = `<div class="UNX-img-wrapper"><img class="UNX-img-block" src="${imgUrl}"/></div>`;
            let priceUI = ``;
            if (price) {
                priceUI = `<span class="UNX-sale-price">${price}</span>`;
            }
            let cardType = ``;
            let descUI = ``;
            let strikeUi = ``;
            if (productViewType === "GRID") {
                cardType = "UNX-grid-card"
            } else {
                cardType = "UNX-list-card";
                if (description) {
                    descUI = `<p class="UNX-description">${description}</p>`;
                }
            }

            return [`<div id="${uniqueId}" data-id="${uniqueId}" data-prank="${idx}" data-item="product" class="UNX-product-col ${cardType} ${productItemClass}">`,
                `<div class="UNX-images-block">${imagesUI}</div>`,
                `<div class="UNX-product-content">`,
                `<h3class="UNX-product-title">${title}</h3>`,
                descUI,
                `<div class="UNX-price-row">${priceUI} ${strikeUi}</div>`,
                `</div>`,
                `</div>`
            ].join('')
        }
    }
});

const createRangeValues = function(prefix, x, y) {
    const start = `${prefix}${x}`;
    const end = `${prefix}${y}`;
    return `<span class="UNX-value-left">${start}</span><span class="UNX-value-right">${end}</span>`
}
window.unbxdSearch.updateConfig({
    products: {
        el: document.getElementById("searchResultsWrapper"),
    },
    spellCheck: {
        enabled: true,
        el: document.getElementById("didYouMeanWrapper")
    },
    noResults: {
        el: document.getElementById("noResultWrapper")
    },
    facet: {
        facetsEl: document.getElementById("facetsWrapper"),
        selectedFacetsEl: document.getElementById("selectedFacetWrapper"),
        onFacetLoad: function(facets) {
            const self = this;
            const {
                facet
            } = this.options;
            const {
                rangeWidgetConfig
            } = facet;
            facets.forEach(facetItem => {
                const {
                    facetType,
                    facetName,
                    gap
                } = facetItem;
                const {
                    prefix
                } = rangeWidgetConfig;

                if (facetType === "range") {
                    const rangeId = `${facetName}_slider`;
                    const sliderElem = document.getElementById(rangeId);
                    const valueEl = document.getElementById(`${rangeId}_value`);
                    let {
                        end,
                        gap,
                        max,
                        min,
                        start
                    } = facetItem;
                    const selectedValues = sliderElem.dataset;
                    if (selectedValues) {
                        start = Number(selectedValues.x),
                            end = Number(selectedValues.y)
                    }
                    this[rangeId] = noUiSlider.create(sliderElem, {
                        start: [start, end],
                        connect: true,
                        range: {
                            'min': 0,
                            'max': max
                        },
                        format: {
                            to: function(value) {
                                return Math.round(value);
                            },
                            from: function(value) {
                                return Math.round(value);
                            }
                        },
                        padding: 0,
                        margin: 0,
                    });
                    this[rangeId].on("set", function(data) {
                        const newData = {
                            start: data[0],
                            end: data[1],
                            facetName,
                            gap
                        };
                        self.setRangeSlider(newData);
                    });
                    this[rangeId].on('slide', function(data) {
                        valueEl.innerHTML = createRangeValues(prefix, data[0], data[1])
                    });

                }

            });
        },
        isSearchable: true,
        defaultOpen: "FIRST",
        rangeTemplate: function(range, selectedRange, facet) {
            const {
                facetName,
                start,
                end,
                gap
            } = range;
            let min = start;
            let max = end;
            const {
                rangeWidgetConfig
            } = facet;
            const {
                prefix
            } = rangeWidgetConfig;
            if (selectedRange.length > 0) {
                const sel = selectedRange[0].replace(/[^\w\s]/gi, '').split(" TO ");
                min = sel[0];
                max = sel[1];
            } else {
                max += gap
            }
            const rangId = `${facetName}_slider`;
            return [`<div id="${facetName}"  data-id="${facetName}" class=" UNX-range-slider-wrap">`,
                `<div id="${rangId}_value"class="UNX-value-container UNX-range-value-block">${createRangeValues(prefix, min, max)}</div>`,
                `<div id="${rangId}" data-x="${min}" data-y="${max}" class="UNX-range-slider-wrapper"></div>`,
                `</div>`,
                `<div>`,
                `</div>`
            ].join('')
        }
    },
    pagination: {
        type: 'FIXED_PAGINATION',
        el: document.querySelectorAll(".unxPagination"),
        pageLimit: 3,
        onPaginate: function() {
            scrollToTop();
        }
    },
    breadcrumb: {
        el: document.getElementById("breadcrumpContainer")
    },
    pagesize: {
        el: document.getElementById("changeNoOfProducts")
    },

    sort: {
        el: document.getElementById("sortWrapper"),
        options: sortOptions
    },
    loader: {
        el: document.getElementById("loaderEl"),
        template: () => {
            return `<div class="UNX-loader"><img src="https://libraries.unbxdapi.com/ss-assets/images/blueLoader.svg" alt="loading image"></div>`
        }
    },
    productView: {
        el: document.getElementById("productViewTypeContainer"),
        template: function(selectedViewType, productViewType) {
            const isDisabled = (this.getSearchResults()) ? false : true;
            const {
                selectedViewTypeClass,
                viewTypeClass
            } = productViewType;
            const {
                UNX_gridBtn,
                UNX_listBtn
            } = this.testIds;
            let listBtnCss = `UNX-list-btn ${viewTypeClass} fa fa-th-list`;
            let gridBtnCss = `UNX-grid-btn ${viewTypeClass} fa fa-th`;
            const gTAttr = `data-test-id=${UNX_gridBtn}`;
            const lTAttr = `data-test-id=${UNX_listBtn}`;
            if (selectedViewType === 'LIST') {
                listBtnCss += ` ${selectedViewTypeClass}`
            }
            if (selectedViewType === 'GRID') {
                gridBtnCss += ` ${selectedViewTypeClass}`
            }
            let listBtn = `<button id="listBtn" class="${listBtnCss}" ${lTAttr} data-view-action="LIST" ></button>`;
            let gridBtn = `<button ${gTAttr} id="gridBtn" class="${gridBtnCss}" data-view-action="GRID" ></button>`;
            if (isDisabled) {
                listBtn = `<button ${lTAttr} id="listBtn" disabled class="${listBtnCss} disabled-btn" data-view-action="LIST" ></button>`;
                gridBtn = `<button ${gTAttr} id="gridBtn" disabled class="${gridBtnCss}  disabled-btn"data-view-action="GRID" ></button>`;
            }
            return `<div class='product-view-container'>${gridBtn} ${listBtn}</div>`

        }
    },
    banner: {
        el: document.getElementById("bannerContainer")
    }
})


const currentQueries = window.unbxdSearch.getQueryParams();
if (!currentQueries.q) {
    window.unbxdSearch.getResults("*");
} else {
    window.unbxdSearch.renderFromUrl()
}
